<?php

namespace seraph_accel;

if( !defined( 'ABSPATH' ) )
	exit;

require_once( __DIR__ . '/Cmn/Gen.php' );
require_once( __DIR__ . '/Cmn/Ui.php' );
require_once( __DIR__ . '/Cmn/Fs.php' );
require_once( __DIR__ . '/Cmn/Db.php' );
require_once( __DIR__ . '/Cmn/Plugin.php' );

const PLUGIN_SETT_VER								= 5;
const PLUGIN_DATA_VER								= 1;
const PLUGIN_EULA_VER								= 1;
const QUEUE_DB_VER									= 2;
const PLUGIN_STAT_VER								= 1;

function GetCacheDir()
{
	return( WP_CONTENT_DIR . '/cache/seraphinite-accelerator' );
}

function GetCacheDataDir( $siteCacheRootPath )
{
	return( $siteCacheRootPath . '/d' );
}

function GetCacheViewsDir( $siteCacheRootPath )
{
	return( $siteCacheRootPath . '/v' );
}

function _GetCacheCurUserSessionHash( $sessionId, $userSessionId, $userId, $expiration )
{
	return( hash_hmac( function_exists( 'hash' ) ? 'sha256' : 'sha1', $sessionId . $userSessionId . $userId . $expiration, AUTH_KEY ) );
}

function GetCacheCurUserSession( $siteId )
{
	global $seraph_accel_g_sessInfo;

	if( is_array( $seraph_accel_g_sessInfo ) )
		return( $seraph_accel_g_sessInfo );

	$sessInfoDef = array( 'userId' => '0' );

	$cookie = Gen::SanitizeTextData( @$_COOKIE[ 'wp_seraph_accel_sess_' . $siteId ] );
	if( empty( $cookie ) )
		return( $seraph_accel_g_sessInfo = $sessInfoDef );

	$cookie_elements = explode( '|', $cookie );
	if( count( $cookie_elements ) !== 5 )
		return( $seraph_accel_g_sessInfo = $sessInfoDef );

	list( $userSessionId, $sessionId, $userId, $expiration, $hmac ) = $cookie_elements;

	if( $expiration && time() > $expiration )
		return( $seraph_accel_g_sessInfo = $sessInfoDef );

	$hmacCheck = _GetCacheCurUserSessionHash( $sessionId, $userSessionId, $userId, $expiration );
	if( $hmac !== $hmacCheck )
		return( $seraph_accel_g_sessInfo = $sessInfoDef );

	return( $seraph_accel_g_sessInfo = array( 'sessId' => $sessionId, 'userSessId' => $userSessionId, 'userId' => $userId, 'expiration' => $expiration ) );
}

function ShouldCurUserSessionExist()
{
	foreach( $_COOKIE as $cookKey => $cookVal )
	{
		if( strpos( $cookKey, 'wordpress_logged_in_' ) === 0 )
			return( true );

	}

	return( false );
}

function SetCacheCurUserSession( $siteId, $sessionId, $userSessionId, $userId, $expiration )
{
	if( headers_sent() )
		return;

	global $seraph_accel_g_sessInfo;

	$hmac = _GetCacheCurUserSessionHash( $sessionId, $userSessionId, $userId, $expiration );
	setcookie( 'wp_seraph_accel_sess_' . $siteId, $userSessionId . '|' . $sessionId . '|' . $userId . '|' . $expiration . '|' . $hmac, Gen::GetCurRequestTime() + YEAR_IN_SECONDS, COOKIEPATH, COOKIE_DOMAIN, false, true );

	$seraph_accel_g_sessInfo = array( 'sessId' => $sessionId, 'userSessId' => $userSessionId, 'userId' => $userId, 'expiration' => $expiration );
}

function _GetDataFileComprExt( $compr, $type )
{

	switch( $compr )
	{
	case 'deflate':
		return( $type == 'html' ? '.deflu' : '.gz' );

	case 'brotli':

		return( '.br' );
	}

	return( '' );
}

function _GetDataFileEncExt( $encoding, $type )
{

	switch( $encoding )
	{
	case 'gzip':
	case 'deflate':
	case 'compress':
		return( $type == 'html' ? '.deflu' : '.gz' );

	case 'br':

		return( '.br' );
	}

	return( '' );
}

function upyiwyheadkxsuxsqsjuw( $tyadkxs )
{
	return( @intval( Gen::GetFileExt( $tyadkxs ), 16 ) );
}

function ujqbvutjqfcestuadkxs_( $data, $level = -1 )
{

	$ctx = deflate_init( ZLIB_ENCODING_RAW, array( 'level' => $level ) );
	if( $ctx === false )
		return( false );

	$data = deflate_add( $ctx, $data, ZLIB_FULL_FLUSH );
	if( $data === false )
		return( false );

	$blockLast = deflate_add( $ctx, '', ZLIB_FINISH );
	if( $blockLast !== "\x03\0" )
		return( false );

	return( $data );
}

function ybjehrjqfcestuadkxs_( $data, $level = 11 )
{

	$ctx = Gen::CallFunc( 'brotli_compress_init', array( $level, BROTLI_GENERIC ), false );
	if( !$ctx )
		return( false );

	$blockFirst = brotli_compress_add( $ctx, '', 1 );
	if( $blockFirst !== "\x6b\x00" )
		return( false );

	$data = brotli_compress_add( $ctx, $data, 1 );
	if( $data === false )
		return( false );

	$blockLast = brotli_compress_add( $ctx, '', BROTLI_FINISH );
	if( $blockLast !== "\x03" )
		return( false );

	return( $data );
}

function IsBrotliAvailable()
{
	return( function_exists( 'brotli_compress_init' ) );
}

function ujyhmadkxsuxsqs( $settCache, $dataPath, $content, $type )
{
	$tyadkxs = @md5( $content ) . '.' . sprintf( '%x', strlen( $content ) );

	$dataComprs = Gen::GetArrField( $settCache, array( 'dataCompr' ), array() );
	if( empty( $dataComprs ) )
		$dataComprs[] = '';

	if( $type != 'html' )
	{
		$dataPath .= '/' . $type;
		if( !in_array( '', $dataComprs, true ) )
			$dataComprs[] = '';
	}

	$writeOk = true;

	foreach( $dataComprs as $dataCompr )
	{
		$dataFileExt = _GetDataFileComprExt( $dataCompr, $type );
		if( $dataFileExt === null )
		{
			$writeOk = false;
			break;
		}

		$ubyvadkxs = $dataPath . '/' . $tyadkxs . '.' . $type . $dataFileExt;
		Gen::FileOpenWithMakeDir( $chunkFileStm, $ubyvadkxs, 'xb' );
		if( $chunkFileStm )
		{
			$contentWrite = false;
			switch( $dataCompr )
			{
			case '':
				$contentWrite = $content;
				break;

			case 'deflate':
				$contentWrite = ( $dataFileExt == '.gz' ) ? @gzencode( $content, 9 ) : ujqbvutjqfcestuadkxs_( $content, 9 );
				break;

			case 'brotli':
				$contentWrite = ( $dataFileExt == '.br' ) ? Gen::CallFunc( 'brotli_compress', array( $content, 11 ), false ) : ybjehrjqfcestuadkxs_( $content, 11 );
				break;
			}

			$writeOk = !( $contentWrite === false || $contentWrite === null ) && ( @fwrite( $chunkFileStm, $contentWrite ) !== false );
			unset( $contentWrite );
			@fclose( $chunkFileStm );

			if( !$writeOk )
				@unlink( $ubyvadkxs );
		}
		else if( !@file_exists( $ubyvadkxs ) )
			$writeOk = false;

		if( !$writeOk )
			break;
	}

	if( !$writeOk )
		return( null );

	if( strpos( $dataPath, ABSPATH ) === 0 )
		$relFilePath = substr( $dataPath, strlen( ABSPATH ) ) . '/';
	else
		$relFilePath = '';
	$relFilePath .= $tyadkxs . '.' . $type;

	return( array( 'id' => $tyadkxs, 'relFilePath' => $relFilePath ) );
}

function CacheReadDsc( $filePath )
{
	return( @unserialize( @file_get_contents( $filePath ) ) );
}

function GetContentProcessorForce( $sett )
{
	if( !@$sett[ 'debug' ] || !isset( $_REQUEST[ 'seraph_accel_proc' ] ) )
		return( null );
	return( !!$_REQUEST[ 'seraph_accel_proc' ] );
}

function ApplyContentProcessorForceSett( &$sett )
{
	$settContPrOverride = Gen::SanitizeTextData( $_REQUEST[ 'seraph_accel_proc' ] );
	if( $settContPrOverride && $settContPrOverride !== '1' )
	{
		$settContPrOverride = @json_decode( @base64_decode( $settContPrOverride ), true );
		if( is_array( $settContPrOverride ) )
		{
			$settContPrOverride[ 'enable' ] = true;
			$sett[ 'contPr' ] = $settContPrOverride;
		}
	}
}

function GetCacheSessionSrvState( $dirSesss )
{
	$ctx = array( 'tmLast' => 0, 'stateId' => '' );

	Gen::DirEnum( $dirSesss . '/st_s', $ctx,
		function( $path, $file, &$ctx )
		{
			if( @filemtime( $path . '/' . $file ) > $ctx[ 'tmLast' ] )
				$ctx[ 'stateId' ] = $file;
		}
	);

	return( $ctx[ 'stateId' ] );
}

function SetCacheSessionSrvState( $dirSesss, $stateId )
{
	$dirSesss = $dirSesss . '/st_s';

	$hr = Gen::MakeDir( $dirSesss );
	if( Gen::HrFail( $hr ) )
		return( $hr );

	@touch( $dirSesss . '/' . $stateId );

	$ctx = array( 'stateId' => $stateId );

	if( !Gen::DirEnum( $dirSesss, $ctx,
		function( $path, $file, &$ctx )
		{
			if( $file != $ctx[ 'stateId' ] )
				@unlink( $path . '/' . $file );
		}
	) )
	{
		return( Gen::E_FAIL );
	}

	return( Gen::S_OK );
}

function GetContCacheEarlySkipData( &$path, &$args )
{
	global $seraph_accel_g_cacheSkipData;

	if( $seraph_accel_g_cacheSkipData !== null )
		return( $seraph_accel_g_cacheSkipData );

	$seraph_accel_g_cacheSkipData = false;

	if( defined( 'SID' ) && SID != '' )
		$seraph_accel_g_cacheSkipData = array( 'skipped', array( 'reason' => 'sid' ) );
	else if( is_admin() )
		$seraph_accel_g_cacheSkipData = array( 'skipped', array( 'reason' => 'admin' ) );
	else if( defined( 'DOING_CRON' ) || isset( $_REQUEST[ 'doing_wp_cron' ] ) )
		$seraph_accel_g_cacheSkipData = array( 'skipped', array( 'reason' => 'cron' ) );
	else if( defined( 'XMLRPC_REQUEST' ) )
		$seraph_accel_g_cacheSkipData = array( 'skipped', array( 'reason' => 'xmlrpc' ) );
	else
	{
		$path = @$_SERVER[ 'REQUEST_URI' ];
		$posQuery = strpos( $path, '?' );

		if( $posQuery !== false )
		{
			$queryOrig = substr( $path, $posQuery + 1 );
			if( $queryOrig != @$_SERVER[ 'QUERY_STRING' ] )
			{
				$args = array();
				@parse_str( $queryOrig, $args );
			}

			$path = substr( $path, 0, $posQuery );
		}
		else
			$args = array();

		$path = strtolower( trim( $path, '/' ) );

		if( strpos( $path, 'robots.txt' ) !== false )
			$seraph_accel_g_cacheSkipData = array( 'skipped', array( 'reason' => 'robots' ) );
		else if( strpos( $path, '.htaccess' ) !== false )
			$seraph_accel_g_cacheSkipData = array( 'skipped', array( 'reason' => 'htaccess' ) );
		else if( strpos( $path, '/wp-' ) !== false )
			$seraph_accel_g_cacheSkipData = array( 'skipped', array( 'reason' => 'wpUrl' ) );
	}

	return( $seraph_accel_g_cacheSkipData );
}

function ContProcGetExclStatus( $siteId, $settCache, $path, &$args, &$varsOut, $adjustArgs = true )
{
	global $seraph_accel_g_contProcGetExclStatus;

	if( $seraph_accel_g_contProcGetExclStatus !== null )
		return( $seraph_accel_g_contProcGetExclStatus );

	$seraph_accel_g_contProcGetExclStatus = false;
	$varsOut = array();

	$tmCur = Gen::GetCurRequestTime();
	$userAgent = strtolower( @$_SERVER[ 'HTTP_USER_AGENT' ] );

	$varsOut[ 'tmCur' ] = $tmCur;
	$varsOut[ 'userAgent' ] = $userAgent;

	$ctxGrps = Gen::GetArrField( $settCache, array( 'ctxGrps' ), array() );

	{

		if( !empty( $args ) )
		{
			if( Gen::GetArrField( $settCache, array( 'exclArgsAll' ), true ) )
			{
				$seraph_accel_g_contProcGetExclStatus = 'exclArgsAll';
				return( $seraph_accel_g_contProcGetExclStatus );
			}

			if( Gen::GetArrField( $settCache, array( 'skipArgsAll' ), false ) )
				$args = array();

			$exclArgs = Gen::GetArrField( $settCache, array( 'exclArgs' ), array() );
			$skipArgs = Gen::GetArrField( $settCache, array( 'skipArgs' ), array() );
			foreach( $args as $argKey => $argVal )
			{
				$argKeyCmp = strtolower( $argKey );

				foreach( $exclArgs as $a )
					if( strpos( $argKeyCmp, $a ) === 0 )
					{
						$seraph_accel_g_contProcGetExclStatus = 'exclArgs:' . $a;
						return( $seraph_accel_g_contProcGetExclStatus );
					}

				foreach( $ctxGrps as $ctxGrp )
				{
					if( !@$ctxGrp[ 'enable' ] )
						continue;

					$ctxArgs = Gen::GetArrField( $ctxGrp, array( 'args' ), array() );
					foreach( $ctxArgs as $a )
						if( strpos( $argKeyCmp, $a ) === 0 )
						{
							$seraph_accel_g_contProcGetExclStatus = 'userCtxArgs:' . $a;
							return( $seraph_accel_g_contProcGetExclStatus );
						}
				}

				if( $adjustArgs )
					foreach( $skipArgs as $a )
						if( strpos( $argKeyCmp, $a ) === 0 )
							unset( $args[ $argKey ] );
			}
		}

		foreach( Gen::GetArrField( $settCache, array( 'urisExcl' ), array() ) as $e )
			if( strpos( $path, $e ) === 0 )
			{
				$seraph_accel_g_contProcGetExclStatus = 'urisExcl:' . $e;
				return( $seraph_accel_g_contProcGetExclStatus );
			}

		foreach( Gen::GetArrField( $settCache, array( 'exclAgents' ), array() ) as $e )
			if( strpos( $userAgent, $e ) !== false )
			{
				$seraph_accel_g_contProcGetExclStatus = 'exclAgents:' . $e;
				return( $seraph_accel_g_contProcGetExclStatus );
			}

		$exclCookies = Gen::GetArrField( $settCache, array( 'exclCookies' ), array() );
		if( $exclCookies )
			foreach( $_COOKIE as $cookKey => $cookVal )
			{
				foreach( $exclCookies as $e )
					if( strpos( $cookKey, $e ) === 0 )
					{
						$seraph_accel_g_contProcGetExclStatus = 'exclCookies:' . $e;
						return( $seraph_accel_g_contProcGetExclStatus );
					}
			}
	}

	$sessInfo = GetCacheCurUserSession( $siteId );

	$userId = @$sessInfo[ 'userId' ];

	$varsOut[ 'sessInfo' ] = $sessInfo;
	$varsOut[ 'userId' ] = $userId;

	{
		$stateCookId = '';
		{
			foreach( $ctxGrps as $ctxGrp )
			{
				if( !@$ctxGrp[ 'enable' ] )
					continue;

				$ctxCookies = Gen::GetArrField( $ctxGrp, array( 'cookies' ), array() );
				foreach( $ctxCookies as $e )
				{
					foreach( $_COOKIE as $cookKey => $cookVal )
					{
						if( strpos( $cookKey, $e ) === 0 )
							$stateCookId .= $cookVal;
					}
				}
			}
		}

		if( ( $stateCookId || $userId )  )
		{
			$varsOut[ 'isUserCtx' ] = true;

				$seraph_accel_g_contProcGetExclStatus = 'userCtx';
		}

		$varsOut[ 'stateCookId' ] = $stateCookId;
	}

	$shouldCurUserSessionExist = ShouldCurUserSessionExist();
	if( ( !@$sessInfo[ 'sessId' ] && ( $shouldCurUserSessionExist || $stateCookId ) ) || ( $shouldCurUserSessionExist && !$userId ) )
		$seraph_accel_g_contProcGetExclStatus = 'noCacheSession';

	return( $seraph_accel_g_contProcGetExclStatus );
}

function ContProcGetSkipStatus( $content )
{
	global $seraph_accel_g_contProcGetSkipStatus;

	if( $seraph_accel_g_contProcGetSkipStatus !== null )
		return( $seraph_accel_g_contProcGetSkipStatus );

	if( defined( 'REST_REQUEST' ) && REST_REQUEST )
		return( $seraph_accel_g_contProcGetSkipStatus = 'restapi' );

	if( defined( 'XMLRPC_REQUEST' ) && XMLRPC_REQUEST )
		return( $seraph_accel_g_contProcGetSkipStatus = 'xmlrpc' );

	if( is_404() )
		return( $seraph_accel_g_contProcGetSkipStatus = '404' );

	$http_response_code = http_response_code();
	if( $http_response_code !== 200 )
		return( $seraph_accel_g_contProcGetSkipStatus = 'httpCode:' . $http_response_code );

	if( is_search() )
		return( $seraph_accel_g_contProcGetSkipStatus = 'search' );
	if( is_feed() )
		return( $seraph_accel_g_contProcGetSkipStatus = 'feed' );

	if( Gen::StrPosArr( $content, array( '</body>', '</BODY>' ) ) === false )
		return( $seraph_accel_g_contProcGetSkipStatus = 'notBody' );

	return( $seraph_accel_g_contProcGetSkipStatus = false );
}

function ContProcIsCompatView( $userAgent  )
{
	global $seraph_accel_g_contProcCompatView;

	if( $seraph_accel_g_contProcCompatView !== null )
		return( $seraph_accel_g_contProcCompatView );

	$compatView = false;

	return( $seraph_accel_g_contProcCompatView = $compatView );
}

function CorrectRequestScheme( &$serverArgs, $target = null )
{

	if( strtolower( @$serverArgs[ 'HTTPS' ] ) == 'on' || ( $target == 'client' && strtolower( @$serverArgs[ 'HTTP_X_FORWARDED_PROTO' ] ) == 'https' ) )
	{
		if( @$serverArgs[ 'REQUEST_SCHEME' ] == 'http' && @$serverArgs[ 'SERVER_PORT' ] == 80 )
			@$serverArgs[ 'SERVER_PORT' ] = 443;
		@$serverArgs[ 'REQUEST_SCHEME' ] = 'https';
	}
	else if( !@$serverArgs[ 'REQUEST_SCHEME' ] )
		$serverArgs[ 'REQUEST_SCHEME' ] = 'http';
}

function _ShouldProcessItemsNext( $remove = false )
{
	$tmRuns = Gen::GetArrfield( get_option( 'seraph_accel_queueProcessItemsNext' ), array( '' ), array() );

	$run = false;
	$tmCur = Gen::GetCurRequestTime();
	for( $i = 0; $i < count( $tmRuns ); $i++ )
	{
		if( $tmCur < $tmRuns[ $i ] )
			continue;

		$run = true;
		if( !$remove )
			break;

		array_splice( $tmRuns, $i--, 1 );
	}

	if( !$run )
		return( false );

	if( $remove )
		update_option( 'seraph_accel_queueProcessItemsNext', $tmRuns );
	return( true );
}

add_filter( 'pre_get_ready_cron_jobs',
	function( $pre )
	{
		if( defined( 'DOING_CRON' ) )
		{

			if( _ShouldProcessItemsNext( true ) )
				_QueueProcessItems();
		}

		return( $pre );
	}
, 0, 1 );

add_action( 'seraph_accel_queueProcessItemsPeriodically',
	function()
	{
		_DelExpiredOption( 'seraph_accel_statusUpdating' );
		_DelExpiredOption( 'seraph_accel_cacheOp' );

		_QueueProcessItems( true );
	}
);

function _QueueProcessItems( $per = false )
{

	$settCache = Gen::GetArrField( Plugin::SettGet(), array( 'cache' ), array() );

	$nMaxItems = @$settCache[ 'maxProc' ];
	if( !$nMaxItems )
		$nMaxItems = 1;
	$nMaxItemsTotal = $nMaxItems;

	$tmCur = microtime( true );

	{
		$items = DbTbl::GetRows( Db::GetTblPrefix( 'seraph_accel_queue' ), null, $nMaxItems, array( 'state' => 1 ) );
		foreach( $items as $item )
		{
			$tmDur = $tmCur - $item[ 'tm' ];
			if( $tmDur <= 300 )
				$nMaxItems --;
			else
			{
				$data = Gen::GetArrField( Gen::Unserialize( @$item[ 'data' ] ), array( '' ), array() );
				$data[ 'p' ] = $item[ 'prior' ];
				$data[ 'td' ] = $tmDur;
				$data[ 'hr' ] = Gen::E_TIMEOUT;
				DbTbl::UpdateRows( Db::GetTblPrefix( 'seraph_accel_queue' ), array( 'state' => 2, 'tm' => $tmCur, 'prior' => 0, 'data' => Gen::Serialize( $item[ 'data' ] ) ), array( 'id' => $item[ 'id' ], 'state' => 1 ) );
			}
		}
	}

	if( $per )
	{
		$items = DbTbl::GetRows( Db::GetTblPrefix( 'seraph_accel_queue' ), array( 'id' ), null, array( 'state' => 2 ), array( 'tm' => 'DESC' ) );
		for( $i = 10; $i < count( $items ); $i++ )
			DbTbl::DeleteRows( Db::GetTblPrefix( 'seraph_accel_queue' ), array( 'id' => $items[ $i ][ 'id' ], 'state' => 2 ) );
	}

	if( !$nMaxItems )
	{

		return;
	}

	$procInterval = @$settCache[ 'procInterval' ];
	if( $procInterval )
	{
		if( $nMaxItems < $nMaxItemsTotal )
			return;

		$procEndLastTime = intval( get_option( 'seraph_accel_procEndLastTime' ) );
		if( $tmCur - $procEndLastTime < $procInterval )
			return;
	}

	$items = DbTbl::GetRows( Db::GetTblPrefix( 'seraph_accel_queue' ), null, $nMaxItems, array( 'state' => 0 ), array( 'prior' => 'ASC', 'tm' => 'ASC' ) );
	if( !$items )
	{

		return;
	}

	foreach( $items as $item )
	{
		$id = @$item[ 'id' ];

		$urlApi = Plugin::GetApiUri( 'CacheProcessItem', array( 'nonce' => hash_hmac( 'md5', '', NONCE_SALT ), 'id' => $id, '_' => microtime( true ) ), null );
		$res = wp_remote_get( $urlApi, array( 'timeout' => 0.01, 'blocking' => false, 'sslverify' => false, 'cookies' => $_COOKIE ) );

	}

}

function CacheInitQueueProcessor()
{
	$time = time() + ( 30 );
	$time -= $time % ( 30 );
	wp_schedule_single_event( $time, 'seraph_accel_queueProcessItemsPeriodically' );

	if( _ShouldProcessItemsNext() )
		wp_schedule_single_event( time(), 'seraph_accel_queueProcessItemsNextPulse' );

}

function CachePushQueueProcessor( $next = false )
{
	$procInterval = 0;
	if( $next )
	{
		$settCache = Gen::GetArrField( Plugin::SettGet(), array( 'cache' ), array() );
		$procInterval = @$settCache[ 'procInterval' ];
	}

	$time = time() + $procInterval;

	$tmRuns = Gen::GetArrfield( get_option( 'seraph_accel_queueProcessItemsNext' ), array( '' ), array() );
	$tmRuns[] = $time;
	update_option( 'seraph_accel_queueProcessItemsNext', $tmRuns );

	if( !$next && defined( 'DISABLE_WP_CRON' ) && DISABLE_WP_CRON )
		return;

	if( !$procInterval )
		wp_remote_get( add_query_arg( array( '_' => microtime( true ) ), site_url( 'wp-cron.php' ) ), array( 'timeout' => 0.01, 'blocking' => false, 'sslverify' => apply_filters( 'https_local_ssl_verify', false ) ) );

}

function OnApi_CachePushQueueProcessor( $args )
{
	if( hash_hmac( 'md5', '', NONCE_SALT ) == Gen::SanitizeTextData( @$args[ 'nonce' ] ) )
		_QueueProcessItems();
}

function OnApi_CacheProcessItem( $args )
{

	if( hash_hmac( 'md5', '', NONCE_SALT ) != Gen::SanitizeTextData( @$args[ 'nonce' ] ) )
		return;

	$id = Gen::SanitizeId( @$args[ 'id' ] );
	if( !$id )
		return;

	$item = DbTbl::GetRows( Db::GetTblPrefix( 'seraph_accel_queue' ), array( 'prior', 'data' ), 1, array( 'id' => $id ) );
	if( !$item )
		return;

	$item = $item[ 0 ];

	$data = Gen::GetArrField( Gen::Unserialize( $item[ 'data' ] ), array( '' ), array() );
	if( !$data )
		return;

	ignore_user_abort( true );
	set_time_limit( 300 + 30 );

	$tmCur = microtime( true );

	if( !DbTbl::UpdateRows( Db::GetTblPrefix( 'seraph_accel_queue' ), array( 'state' => 1, 'tm' => $tmCur ), array( 'id' => $id, 'state' => 0 ) ) )
		return;

	$url = add_query_arg( array( 'seraph_accel_prep' => @rawurlencode( @base64_encode( @json_encode( array( 'nonce' => hash_hmac( 'md5', '', NONCE_SALT ), 'force' => @$data[ 'f' ], '_tm' => $tmCur ) ) ) ) ), @$data[ 'u' ] );

	$requestRes = Wp::RemoteGet( $url, array( 'timeout' => 300, 'sslverify' => false, 'headers' => @$data[ 'h' ] ) );
	$tmFinish = microtime( true );
	$data[ 'td' ] = $tmFinish - $tmCur;
	$data[ 'p' ] = $item[ 'prior' ];

	$hr = Net::GetHrFromWpRemoteGet( $requestRes, true );
	if( $hr != Gen::S_OK )
		$data[ 'hr' ] = $hr;

	DbTbl::UpdateRows( Db::GetTblPrefix( 'seraph_accel_queue' ), array( 'state' => 2, 'tm' => $tmFinish, 'prior' => 0, 'data' => Gen::Serialize( $data ) ), array( 'id' => $id, 'state' => 1 ) );
	update_option( 'seraph_accel_procEndLastTime', ( int )$tmFinish );
	CachePushQueueProcessor( true );
}

function CacheInitQueueTable( $force = false )
{
	$data = Plugin::DataGet();
	if( !$force && Gen::GetArrField( $data, 'queueDbVer', 0 ) == QUEUE_DB_VER )
		return;

	DbTbl::CreateUpdate( Db::GetTblPrefix( 'seraph_accel_queue' ), array(
		'id' => array( 'type' => 'CHAR(32)', 'attrs' => array( 'NOT NULL', 'PRIMARY KEY' ) ),
		'prior' => array( 'type' => 'INT', 'attrs' => array( 'INDEX' ) ),
		'state' => array( 'type' => 'INT', 'attrs' => array( 'INDEX' ) ),
		'tm' => array( 'type' => 'DOUBLE', 'attrs' => array( 'INDEX' ) ),
		'data' => array( 'type' => 'TEXT' ),
	) );

	Gen::SetArrField( $data, 'queueDbVer', QUEUE_DB_VER );
	Plugin::DataSet( $data );
}

function CachePostPreparePageEx( $url, $priority, $headers = null, $force = false )
{
	$settCache = Gen::GetArrField( Plugin::SettGet(), array( 'cache' ), array() );

	if( !is_array( $headers ) )
		$headers = array();

	$id = $url . $priority;
	foreach( $headers as $header => $headerVal )
		$id .= $header . $headerVal;
	$id = md5( $id );

	$viewName = null;
	if( $viewsDeviceGrp = GetCacheViewDeviceGrp( $settCache, strtolower( @$headers[ 'User-Agent' ] ) ) )
	{
		$viewName = @$viewsDeviceGrp[ 'name' ];
		if( !$viewName )
			$viewName = 'id:' . @$viewsDeviceGrp[ 'id' ];
	}

	DbTbl::DeleteRows( Db::GetTblPrefix( 'seraph_accel_queue' ), array( 'id' => $id, 'state' => 2 ) );
	$res = !!DbTbl::InsertRow( Db::GetTblPrefix( 'seraph_accel_queue' ), array( 'id' => $id, 'state' => 0, 'prior' => $priority, 'tm' => microtime( true ), 'data' => Gen::Serialize( array( 'u' => $url, 'f' => $force, 'h' => $headers, 'v' => $viewName ) ) ) );

	return( $res );
}

function CachePostPreparePage( $url, $priority, $headers = null, $force = false )
{
	if( CachePostPreparePageEx( $url, $priority, $headers, $force ) )
		CachePushQueueProcessor();
}

function CachePrepareQueueDelete()
{
	DbTbl::DeleteRows( Db::GetTblPrefix( 'seraph_accel_queue' ), array( 'state' => 0 ) );
}

function GetExtContents( $url, $userAgentCmn = true, $timeout = 30 )
{
	$args = array( 'sslverify' => true, 'timeout' => $timeout );
	if( $userAgentCmn )
		$args[ 'user-agent' ] = 'Mozilla/99999.9 AppleWebKit/9999999.99 (KHTML, like Gecko) Chrome/999999.0.9999.99 Safari/9999999.99';

	$res = Wp::RemoteGet( $url, $args );
	if( is_wp_error( $res ) )
		return( false );

	$code = wp_remote_retrieve_response_code( $res );
	if( !( $code >= 200 && $code < 400 ) )
		return( false );

	return( wp_remote_retrieve_body( $res ) );
}

function CacheExtractPreparePageParams( &$args )
{
	$prms = @$args[ 'seraph_accel_prep' ];
	if( !$prms )
		return( null );

	$requestUri = &$_SERVER[ 'REQUEST_URI' ];
	$requestUriArgs = Net::UrlExtractArgs( $requestUri );

	$redirect_query_string_args = Net::UrlParseQuery( $_SERVER[ 'REDIRECT_QUERY_STRING' ] );
	$query_string_args = Net::UrlParseQuery( $_SERVER[ 'QUERY_STRING' ] );

	unset( $args[ 'seraph_accel_prep' ] );
	unset( $_GET[ 'seraph_accel_prep' ] );
	unset( $_REQUEST[ 'seraph_accel_prep' ] );
	unset( $requestUriArgs[ 'seraph_accel_prep' ] );
	unset( $redirect_query_string_args[ 'seraph_accel_prep' ] );
	unset( $query_string_args[ 'seraph_accel_prep' ] );

	$requestUri = Net::UrlAddArgs( $requestUri, $requestUriArgs );

	$_SERVER[ 'REDIRECT_QUERY_STRING' ] = Net::UrlBuildQuery( $redirect_query_string_args );
	$_SERVER[ 'QUERY_STRING' ] = Net::UrlBuildQuery( $query_string_args );

	$prms = @json_decode( @base64_decode( Gen::SanitizeTextData( $prms ) ), true );
	if( hash_hmac( 'md5', '', NONCE_SALT ) != $prms[ 'nonce' ] )
		return( null );

	unset( $prms[ 'nonce' ] );
	return( $prms );
}

function GetCacheViewDeviceGrp( $settCache, $userAgent )
{
	if( !@$settCache[ 'views' ] )
		return( null );

	$viewsDeviceGrps = Gen::GetArrField( $settCache, array( 'viewsDeviceGrps' ), array() );
	foreach( $viewsDeviceGrps as $viewsDeviceGrp )
	{
		if( !@$viewsDeviceGrp[ 'enable' ] )
			continue;

		foreach( Gen::GetArrField( $viewsDeviceGrp, array( 'agents' ), array() ) as $a )
			if( strpos( $userAgent, $a ) !== false )
				return( $viewsDeviceGrp );
	}

	return( null );
}

function _SetExpirableOption( $option, $value, $timeout )
{
	update_option( $option, $value );
	update_option( $option . '_tmMax', time() + $timeout );
}

function _DelExpirableOption( $option )
{
	delete_option( $option );
	delete_option( $option . '_tmMax' );
}

function _DelExpiredOption( $option )
{
	$tmMax = get_option( $option . '_tmMax' );
	if( $tmMax && $tmMax < time() )
		_DelExpirableOption( $option );
}

function LogWrite( $text, $severity = Ui::MsgInfo )
{
	Gen::FileOpenWithMakeDir( $stm, GetCacheDir() . '/logs/log.txt', 'a' );
	if( !$stm )
		return;

	switch( $severity )
	{
	case Ui::MsgSucc:	$severity = 'S'; break;
	case Ui::MsgWarn:	$severity = 'W'; break;
	case Ui::MsgErr:	$severity = 'E'; break;
	default:			$severity = 'I'; break;
	}

	@fwrite( $stm, gmdate( 'd M Y H:i:s', time() ) . ' GMT <' . $severity . '>' . "\t" . $text . "\r\n" );
	@fclose( $stm );
}

