<?php

namespace seraph_accel;

if( !defined( 'ABSPATH' ) )
	exit;

function _CacheDirWalk_User( $path, $item, &$ctx )
{
	$path = $path . '/' . $item . '/s';
	if( !@is_dir( $path ) )
		return( true );

	return( Gen::DirEnum( $path, $ctx,
		function( $path, $item, &$ctx )
		{
			$path = $path . '/' . $item . '/s';
			if( !@is_dir( $path ) )
				return( true );

			$ctx[ 'isDefSess' ] = ( $item == '@' );

			return( Gen::DirEnum( $path, $ctx,
				function( $path, $item, &$ctx )
				{
					$path = $path . '/' . $item . '/o';
					if( !@is_dir( $path ) )
						return( true );

					$ctx[ 'isDefState' ] = ( $item == '@' );

					$recurse = true;
					$objPathSpec = Gen::GetArrField( $ctx[ 'spec' ], array( 'objPath' ) );
					if( $objPathSpec !== null )
					{
						if( $objPathSpec )
							$path .= '/' . $objPathSpec;
						$recurse = false;
					}

					return( Gen::DirEnum( $path, $ctx,
						function( $path, $item, &$ctx )
						{
							$item = $path . '/' . $item;
							if( @is_dir( $item ) )
								return( true );
							return( call_user_func_array( $ctx[ 'cbObj' ], array( &$ctx[ 'ctxWalk' ], !( $ctx[ 'isDefSess' ] && $ctx[ 'isDefState' ] ), $item ) ) );
						}
						, $recurse
					) );
				}
			) );
		}
	) );
}

function _CacheDirWalk( $siteId, &$ctxWalk, $spec = null, $cbObj = null, $cbData = null )
{
	$siteCacheRootPath = GetCacheDir() . '/s/' . $siteId;

	$ctx = array( 'ctxWalk' => &$ctxWalk, 'spec' => $spec, 'cbObj' => $cbObj, 'cbData' => $cbData );

	if( $cbObj )
	{
		if( Gen::DirEnum( GetCacheViewsDir( $siteCacheRootPath ), $ctx,
			function( $path, $item, &$ctx )
			{
				$path = $path . '/' . $item . '/c';
				if( !@is_dir( $path ) )
					return( true );

				$userIdSpec = Gen::GetArrField( $ctx[ 'spec' ], array( 'userId' ) );
				if( $userIdSpec )
					return( _CacheDirWalk_User( $path, '' . $userIdSpec, $ctx ) );
				return( Gen::DirEnum( $path, $ctx, 'seraph_accel\\_CacheDirWalk_User' ) );
			}
		) === false )
		{
			return( false );
		}
	}

	if( $cbData )
	{
		$cacheDataDir = GetCacheDataDir( $siteCacheRootPath );
		$ctx[ 'cacheDataDirLen' ] = strlen( $cacheDataDir );

		if( Gen::DirEnum( $cacheDataDir, $ctx,
			function( $path, $item, &$ctx )
			{
				$itemId = $item;

				$item = $path . '/' . $item;
				if( @is_dir( $item ) )
					return( true );

				$itemId = explode( '.', $itemId );
				$itemId = '' . @$itemId[ 0 ] . '.' . @$itemId[ 1 ];

				$itemType = substr( $path, $ctx[ 'cacheDataDirLen' ] + 1 );
				return( call_user_func_array( $ctx[ 'cbData' ] , array( &$ctx[ 'ctxWalk' ], $itemType ? $itemType : 'html', $itemId, $item ) ) );
			}
			, true
		) === false )
		{
			return( false );
		}
	}

	return( true );
}

function CacheGetInfo( $siteId, $cbCancel )
{
	$info = array( 'cbCancel' => $cbCancel, 'nObj' => 0, 'nJs' => 0, 'nCss' => 0, 'size' => 0, 'sizeUncompr' => 0, 'sizeObjFrag' => 0, 'sizeObj' => 0 );

	if( _CacheDirWalk( $siteId, $info, null,
		function( &$info, $isUserCtx, $objFile )
		{
			if( $info[ 'cbCancel' ]() )
				return( false );

			$info[ 'nObj' ] += 1;
			$sz = @filesize( $objFile );
			$info[ 'size' ] += $sz;
			$info[ 'sizeUncompr' ] += $sz;

			foreach( Gen::GetArrField( CacheReadDsc( $objFile ), array( 'p' ), array() ) as $tyadkxs )
			{
				if( $info[ 'cbCancel' ]() )
					return( false );

				$sz = upyiwyheadkxsuxsqsjuw( $tyadkxs );

				$info[ 'sizeObj' ] += $sz;

			}
		},

		function( &$info, $dataType, $dataId, $dataFile )
		{
			if( $info[ 'cbCancel' ]() )
				return( false );

			$sz = @filesize( $dataFile );
			$info[ 'size' ] += $sz;

			if( $dataType == Gen::GetFileExt( $dataFile ) )
			{
				switch( $dataType )
				{
				case 'js': $info[ 'nJs' ] += 1; break;
				case 'css': $info[ 'nCss' ] += 1; break;
				}

				$info[ 'sizeUncompr' ] += $sz;
			}
			else
				$info[ 'sizeUncompr' ] += upyiwyheadkxsuxsqsjuw( Gen::GetFileName( Gen::GetFileName( $dataFile, true ), true ) );

			if( $dataType == 'html' )
			{
				$fileNameCount = null;
				if( $dataType == Gen::GetFileExt( $dataFile ) )
					$fileNameCount = $dataFile;
				else
				{
					$dataFileUncompr = Gen::GetFileName( $dataFile, true, true );
					if( $dataType == Gen::GetFileExt( $dataFileUncompr ) && !@file_exists( $dataFileUncompr ) )
						$fileNameCount = $dataFileUncompr;
				}

				if( $fileNameCount )
					$info[ 'sizeObjFrag' ] += upyiwyheadkxsuxsqsjuw( Gen::GetFileName( $fileNameCount, true ) );
			}
		}
	) === false )
	{
		return( null );
	}

	unset( $info[ 'cbCancel' ] );
	return( $info );
}

class CacheUrlProcessor
{
	private $viewsHeaders;
	private $priority;

	public function __construct( $settCache, $priority )
	{
		$this -> viewsHeaders = CacheOpGetViewsHeaders( $settCache );
		$this -> priority = $priority;
	}

	function __destruct()
	{
	}

	function isAbort()
	{
		return( get_option( 'seraph_accel_cacheOp' ) === false );
	}

	function op( $url )
	{

		foreach( $this -> viewsHeaders as $headers )
		{
			if( $this -> isAbort() )
				return( false );

			if( CachePostPreparePageEx( $url, $this -> priority, $headers ) )
			{

			}
			usleep( 1 );
		}
	}
}

function CacheOp( $siteId, $op, $priority = 0 )
{
	if( get_option( 'seraph_accel_cacheOp' ) !== false )
		return( false );

	_SetExpirableOption( 'seraph_accel_cacheOp', $op, 1800 );

	$optDropCache = new WpOptionDropCache( 'seraph_accel_cacheOp', 0.5 );

	$ctx = array( 'op' => $op, 'datasDel' => array() );

	if( $op == 1 )
	{
		$sett = Plugin::SettGet();
		$ctx[ 'timeout' ] = Gen::GetArrField( $sett, array( 'cache', 'timeoutCln' ), 0 ) * 60;
		$ctx[ 'timeoutCtx' ] = Gen::GetArrField( $sett, array( 'cache', 'ctxTimeoutCln' ), 0 ) * 60;
		$ctx[ 'tmCur' ] = Gen::GetCurRequestTime();
		unset( $sett );

		if( _CacheDirWalk( $siteId, $ctx, null,
			null,
			function( &$ctx, $dataType, $dataId, $dataFile )
			{
				if( get_option( 'seraph_accel_cacheOp' ) === false )
					return( false );

				$ctx[ 'datasDel' ][ $dataType ][ $dataId ] = true;
			}
		) === false )
		{
			return( false );
		}
	}

	if( _CacheDirWalk( $siteId, $ctx, null,
		function( &$ctx, $isUserCtx, $objFile )
		{
			if( get_option( 'seraph_accel_cacheOp' ) === false )
				return( false );

			if( $ctx[ 'op' ] == 0 )
			{
				@touch( $objFile, 0 );
				return;
			}

			if( $ctx[ 'op' ] == 2 )
			{
				@unlink( $objFile );
				return;
			}

			{
				$dscFileTm = @filemtime( $objFile );
				if( $dscFileTm > 0 )
				{
					$dscFileTmAge = $ctx[ 'tmCur' ] - $dscFileTm;
					$timeout = $isUserCtx ? $ctx[ 'timeoutCtx' ] : $ctx[ 'timeout' ];

					if( $timeout > 0 && $dscFileTmAge > $timeout )
					{
						@unlink( $objFile );
						return;
					}
				}
			}

			$dsc = CacheReadDsc( $objFile );
			if( !$dsc )
			{

				return;
			}

			$chunks = @$dsc[ 'p' ];
			if( !is_array( $chunks ) )
			{

				return;
			}

			foreach( $chunks as $tyadkxs )
				unset( $ctx[ 'datasDel' ][ 'html' ][ $tyadkxs ] );

			$childrenTypes = @$dsc[ 's' ];
			if( $childrenTypes !== null )
			{
				if( !is_array( $childrenTypes ) )
				{

					return;
				}

				foreach( $childrenTypes as $childType => $children )
					foreach( $children as $childId )
						unset( $ctx[ 'datasDel' ][ $childType ][ $childId ] );
			}
		},

		$op != 2 ? null : function( &$ctx, $dataType, $dataId, $dataFile )
		{
			if( get_option( 'seraph_accel_cacheOp' ) === false )
				return( false );

			@unlink( $dataFile );
		}
	) === false )
	{
		return( false );
	}

	if( $op == 1 )
	{
		foreach( $ctx[ 'datasDel' ] as $datasDelType => $datasDel )
			if( empty( $datasDel ) )
				unset( $ctx[ 'datasDel' ][ $datasDelType ] );

		if( $ctx[ 'datasDel' ] )
			if( _CacheDirWalk( $siteId, $ctx, null,
				null,
				function( &$ctx, $dataType, $dataId, $dataFile )
				{
					if( get_option( 'seraph_accel_cacheOp' ) === false )
						return( false );

					if( @$ctx[ 'datasDel' ][ $dataType ][ $dataId ] )
						@unlink( $dataFile );
				}
			) === false )
			{
				return( false );
			}
	}

	if( $op == 1 || $op == 2 )
	{
		if( Gen::DirEnum( GetCacheViewsDir( GetCacheDir() . '/s/' . $siteId ), $ctx,
			function( $path, $item, &$ctx )
			{
				if( get_option( 'seraph_accel_cacheOp' ) === false )
					return( false );

				$path = $path . '/' . $item;
				if( @is_dir( $path ) )
					@rmdir( $path );
			}
			, true
		) === false )
		{
			return( false );
		}
	}

	$settCache = Gen::GetArrField( Plugin::SettGet(), array( 'cache' ), array() );
	if( ( $op == 0 || $op == 2 ) && @$settCache[ 'autoProc' ] )
	{
		CachePrepareQueueDelete();

		$proc = new CacheUrlProcessor( $settCache, $priority );
		if( EnumSiteUrls( $proc, $siteId ) === false )
			return( false );

		CachePushQueueProcessor();
	}

	if( get_option( 'seraph_accel_cacheOp' ) === false )
		return( false );

	_DelExpirableOption( 'seraph_accel_cacheOp' );
	return( true );
}

function EnumSiteUrls( $proc, $siteId )
{

	$siteUrl = get_site_url( null );

	if( $proc -> op( $siteUrl ) === false )
		return( false );

	if( EnumSitemapUrls( $proc, rtrim( $siteUrl, '/' ) . '/sitemap.xml' ) === false )
		return( false );

	return( true );
}

function EnumSitemapUrls( $proc, $urlSitemap, $tryLocal = true )
{

	$data = @GetExtContents( $urlSitemap );
	if( $proc -> isAbort() )
		return( false );

	if( !$data && $tryLocal )
		$data = @file_get_contents( rtrim( get_home_path(), '/' ) . '/sitemap.xml' );

	if( !$data )
		return( null );

	if( $proc -> isAbort() )
		return( false );

	$doc = new \DOMDocument();
	$doc -> strictErrorChecking = false;

	if( !@$doc -> loadXML( $data, LIBXML_BIGLINES | LIBXML_NONET | LIBXML_PARSEHUGE ) )
		return( null );

	foreach( $doc -> getElementsByTagName( 'sitemap' ) as $child )
		foreach( $child -> getElementsByTagName( 'loc' ) as $loc )
			if( EnumSitemapUrls( $proc, $loc -> nodeValue, false ) === false )
				return( false );

	foreach( $doc -> getElementsByTagName( 'url' ) as $child )
		foreach( $child -> getElementsByTagName( 'loc' ) as $loc )
			if( $loc -> tagName == 'loc' && $proc -> op( $loc -> nodeValue ) === false )
				return( false );

	return( true );
}

function CacheOpUser( $userId, $op )
{
	$siteId = 'm';

	$ctx = array( 'op' => $op );

	_CacheDirWalk( $siteId, $ctx, array( 'userId' => $userId ),
		function( &$ctx, $isUserCtx, $objFile )
		{
			if( $ctx[ 'op' ] == 0 )
			{
				@touch( $objFile, 0 );
				return;
			}

			if( $ctx[ 'op' ] == 2 )
			{
				@unlink( $objFile );
				return;
			}
		}
	);

}

function CacheOpPost( $postId, $op, $priority = 0 )
{
    $post = get_post( $postId );
	if( !$post )
		return;

	if( !in_array( $post -> post_status, array( 'publish' ) ) )
		return;

	$siteId = 'm';

	$uris = array( trim( Net::Url2Uri( get_permalink( $postId ), true ), '/' ) );

	$dependUrls = Gen::GetArrField( Plugin::SettGet(), array( 'cache', 'updPostDeps' ), array() );
	foreach( $dependUrls as $dependUrl )
		$uris[] = trim( Net::Url2Uri( $dependUrl, true ), '/' );

	if( $op !== 0 )
	{
		CacheOpUris( $siteId, $uris[ 0 ], $op, $priority );
		array_splice( $uris, 0, 1 );
	}

	CacheOpUris( $siteId, $uris, 0, $priority );
}

function CacheOpCancel( $siteId )
{
	delete_option( 'seraph_accel_cacheOp' );
}

function CacheGetCurOp( $siteId )
{
	return( get_option( 'seraph_accel_cacheOp' ) );
}

function CacheOpUris( $siteId, $uris, $op, $priority = 0 )
{
	if( get_option( 'seraph_accel_cacheOp' ) !== false )
		return( false );

	if( !is_array( $uris ) )
		$uris = array( $uris );

	_SetExpirableOption( 'seraph_accel_cacheOp', $op, 1800 );

	$optDropCache = new WpOptionDropCache( 'seraph_accel_cacheOp', 0.5 );

	$ctx = array( 'op' => $op );

	foreach( $uris as $uri )
	{
		if( _CacheDirWalk( $siteId, $ctx, array( 'objPath' => $uri ),
			function( &$ctx, $isUserCtx, $objFile )
			{
				if( get_option( 'seraph_accel_cacheOp' ) === false )
					return( false );

				if( $ctx[ 'op' ] == 0 )
				{
					@touch( $objFile, 0 );
					return;
				}

				if( $ctx[ 'op' ] == 2 )
				{
					@unlink( $objFile );
					return;
				}
			}
		) === false )
		{
			return( false );
		}
	}

	$settCache = Gen::GetArrField( Plugin::SettGet(), array( 'cache' ), array() );
	if( @$settCache[ 'autoProc' ] )
	{
		$proc = new CacheUrlProcessor( $settCache, $priority );

		$siteAddr = trim( get_site_url( null ), '/' );
		foreach( $uris as $uri )
		{
			if( get_option( 'seraph_accel_cacheOp' ) === false )
				return( false );

			$url = $siteAddr . '/' . $uri;
			if( $proc -> op( $url ) === false )
				return( false );
		}

		CachePushQueueProcessor();
	}

	_DelExpirableOption( 'seraph_accel_cacheOp' );
	return( true );
}

function CacheOpGetViewsHeaders( $settCache )
{
	$res = array( array( 'User-Agent' => 'seraph-accel-Agent/1.8.7' ) );

	if( !@$settCache[ 'views' ] )
		return( $res );

	$viewsDeviceGrps = Gen::GetArrField( $settCache, array( 'viewsDeviceGrps' ), array() );
	foreach( $viewsDeviceGrps as $viewsDeviceGrp )
	{
		if( !@$viewsDeviceGrp[ 'enable' ] )
			continue;

		if( $agents = Gen::GetArrField( $viewsDeviceGrp, array( 'agents' ), array() ) )
			$res[] = array( 'User-Agent' => 'seraph-accel-Agent/1.8.7 (' . $agents[ 0 ] . ')' );
	}

	return( $res );
}

function CacheVerifyEnvDropin( $sett )
{
	return( @file_get_contents( WP_CONTENT_DIR . '/advanced-cache.php' ) == GetAdvCacheFileContent( $sett ) );
}

function CacheInitEnvDropin( $sett, $init = true )
{
	$cont = @file_get_contents( WP_CONTENT_DIR . '/advanced-cache.php' );

	if( !$init )
	{
		if( $cont && strpos( $cont, '/* seraphinite-accelerator */' ) !== false )
			@file_put_contents( WP_CONTENT_DIR . '/advanced-cache.php', '' );
		return( Gen::S_OK );
	}

	$contNew = GetAdvCacheFileContent( $sett );
	if( $cont != $contNew )
		$hr = Gen::HrAccom( $hr, @file_put_contents( WP_CONTENT_DIR . '/advanced-cache.php', $contNew ) !== false ? Gen::S_OK : Gen::E_FAIL );
	return( $hr );
}

function IsWpCacheActive()
{
	return( defined( 'WP_CACHE' ) && WP_CACHE );
}

function CacheInitEnv( $sett, $init = true )
{
	$cacheEnable = Gen::GetArrField( $sett, 'cache/enable', true, '/' );

	if( !$cacheEnable || !$init )
	{
		CacheInitEnvDropin( $sett, false );
		wp_clear_scheduled_hook( 'seraph_accel_autoclean' );
	}

	if( !$init )
	{
		if( Gen::HtAccess_IsSupported() )
			Gen::HtAccess_SetBlock( 'seraphinite-accelerator', '' );

		return( Gen::S_OK );
	}

	if( $cacheEnable )
	{
		$hr = Gen::HrAccom( $hr, CacheInitEnvDropin( $sett ) );
		if( !IsWpCacheActive() )
			$hr = Gen::HrAccom( $hr, Php::File_SetDefineVal( Wp::GetConfigFilePath(), 'WP_CACHE', true ) );
		$hr = Gen::HrAccom( $hr, Gen::MakeDir( GetCacheDir(), true ) );

		if( @$sett[ 'autoCleaning' ] )
			wp_schedule_event( time() + 60, 'hourly', 'seraph_accel_autoclean' );
		else
			wp_clear_scheduled_hook( 'seraph_accel_autoclean' );
	}

	if( Gen::HtAccess_IsSupported() )
	{
		$htaccessBlock = '';

		if( Gen::GetArrField( $sett, 'cacheBr/enable', false, '/' ) )
		{
			$tmStr = '"access plus ' . Gen::GetArrField( $sett, 'cacheBr/timeout', 0, '/' ) . ' minutes"';

			$htaccessBlock .=
				'<IfModule mod_expires.c>' . "\n\t" . 'ExpiresActive on' . "\n\t" . 'ExpiresByType text/css                      ' . $tmStr . "\n\t" . 'ExpiresByType text/javascript               ' . $tmStr . "\n\t" . 'ExpiresByType application/javascript        ' . $tmStr . "\n\t" . 'ExpiresByType application/x-javascript      ' . $tmStr . "\n\t" . 'ExpiresByType font/eot                      ' . $tmStr . "\n\t" . 'ExpiresByType font/opentype                 ' . $tmStr . "\n\t" . 'ExpiresByType font/woff                     ' . $tmStr . "\n\t" . 'ExpiresByType application/vnd.ms-fontobject ' . $tmStr . "\n\t" . 'ExpiresByType application/font-woff         ' . $tmStr . "\n\t" . 'ExpiresByType application/font-woff2        ' . $tmStr . "\n\t" . 'ExpiresByType application/x-font-ttf        ' . $tmStr . "\n\t" . 'ExpiresByType application/x-font-woff       ' . $tmStr . "\n\t" . 'ExpiresByType image/vnd.microsoft.icon      ' . $tmStr . "\n\t" . 'ExpiresByType image/x-icon                  ' . $tmStr . "\n\t" . 'ExpiresByType image/bmp                     ' . $tmStr . "\n\t" . 'ExpiresByType image/gif                     ' . $tmStr . "\n\t" . 'ExpiresByType image/jpeg                    ' . $tmStr . "\n\t" . 'ExpiresByType image/png                     ' . $tmStr . "\n\t" . 'ExpiresByType image/svg+xml                 ' . $tmStr . "\n\t" . 'ExpiresByType image/webp                    ' . $tmStr . "\n\t" . 'ExpiresByType audio/ogg                     ' . $tmStr . "\n\t" . 'ExpiresByType video/mp4                     ' . $tmStr . "\n\t" . 'ExpiresByType video/ogg                     ' . $tmStr . "\n\t" . 'ExpiresByType video/webm                    ' . $tmStr . "\n" .
				'</IfModule>' . "\n" .

				'<IfModule mod_headers.c>' . "\n\t" . 'Header unset ETag' . "\n" .
				'</IfModule>' . "\n" .
				'FileETag None' . "\n" .
				'';
		}

		if( Gen::GetArrField( $sett, 'contPr/img/redirWebp', false, '/' ) )
		{
			$htaccessBlock .=
				'<IfModule mod_rewrite.c>' . "\n\t" . 'RewriteEngine On' . "\n\t" . 'RewriteCond %{HTTP_ACCEPT} image\\/webp' . "\n\t" . 'RewriteCond %{REQUEST_FILENAME} \\.(jpe|jpg|jpeg|png|gif)$' . "\n\t" . 'RewriteCond %{REQUEST_FILENAME}.webp -f' . "\n\t" . 'RewriteRule ^(.*)\\.(jpe|jpg|jpeg|png|gif)$ $1\\.$2\\.webp [QSA]' . "\n" .
				'</IfModule>' . "\n" .
				'<IfModule mod_headers.c>' . "\n\t" . '<FilesMatch \\.(jpe|jpg|jpeg|png|gif)\\.webp$>' . "\n\t\t" . 'Header merge Vary Accept' . "\n" .
				( Gen::GetArrField( $sett, array( 'hdrTrace' ), false ) ? "\t\t" . 'Header set X-seraph-accel-Cache "1.8.7; state=preoptimized;"' . "\n" : '' ) .
				"\t" . '</FilesMatch>' . "\n" .
				'</IfModule>' . "\n" .

				'';
		}

		{
			$encs = Gen::GetArrField( $sett, array( 'cache', 'encs' ), array() );
			$mimeTypes = array(
				'text/plain',

				'text/css',

				'text/javascript',
				'application/javascript',
				'application/x-javascript',
				'application/json',

				'text/html',
				'text/xml',
				'application/atom+xml',
				'application/rss+xml',
				'application/xhtml+xml',
				'application/xml',
				'text/x-component',

				'application/vnd.ms-fontobject',
				'application/x-font-ttf',
				'font/eot',
				'font/opentype',

				'image/bmp',
				'image/svg+xml',
				'image/vnd.microsoft.icon',
				'image/x-icon',
			);

			if( in_array( 'br', $encs ) )
			{
				$htaccessBlock .=
					'<IfModule mod_brotli.c>' . "\n\t" . '<IfModule mod_filter.c>' . "\n\t\t" . 'AddOutputFilterByType BROTLI_COMPRESS';

				foreach( $mimeTypes as $mimeType )
					$htaccessBlock .= ' ' . $mimeType;

				$htaccessBlock .=
					"\n\t" . '</IfModule>' . "\n" .
					'</IfModule>' . "\n" .
					'';
			}

			if( in_array( 'gzip', $encs ) || in_array( 'deflate', $encs ) || in_array( 'compress', $encs ) )
			{
				$htaccessBlock .=
					'<IfModule mod_deflate.c>' . "\n\t" . '<IfModule mod_filter.c>' . "\n\t\t" . 'AddOutputFilterByType DEFLATE';

				foreach( $mimeTypes as $mimeType )
					$htaccessBlock .= ' ' . $mimeType;

				$htaccessBlock .=
					"\n\t" . '</IfModule>' . "\n" .
					'</IfModule>' . "\n" .
					'';
			}
		}

		{
			$dataComprs = Gen::GetArrField( $sett, array( 'cache', 'dataCompr' ), array() );

			if( in_array( 'brotli', $dataComprs ) )
			{
				$htaccessBlock .=
					'<IfModule mod_rewrite.c>' . "\n\t" . 'RewriteEngine On' . "\n\t" . 'RewriteCond %{HTTP:Accept-Encoding} (^|\\W)br(\\W|$)' . "\n\t" . 'RewriteCond %{REQUEST_FILENAME} \\.(css|js)$' . "\n\t" . 'RewriteCond %{REQUEST_FILENAME}.br -f' . "\n\t" . 'RewriteRule ^(.*)\\.(css|js)$ $1\\.$2\\.br [QSA]' . "\n\t" . 'RewriteRule \\.css\\.br$ - [T=text/css,E=no-gzip:1,E=no-brotli:1]' . "\n\t" . 'RewriteRule \\.js\\.br$ - [T=application/javascript,E=no-gzip:1,E=no-brotli:1]' . "\n" .
					'</IfModule>' . "\n" .
					'<IfModule mod_headers.c>' . "\n\t" . '<FilesMatch \\.(js|css)\\.br$>' . "\n\t\t" . 'Header set Content-Encoding br' . "\n\t\t" . 'Header merge Vary Accept-Encoding' . "\n" .
					( Gen::GetArrField( $sett, array( 'hdrTrace' ), false ) ? "\t\t" . 'Header set X-seraph-accel-Cache "1.8.7; state=precompressed;"' . "\n" : '' ) .
					"\t" . '</FilesMatch>' . "\n" .
					'</IfModule>' . "\n" .
					'';
			}

			if( in_array( 'deflate', $dataComprs ) )
			{
				$htaccessBlock .=
					'<IfModule mod_rewrite.c>' . "\n\t" . 'RewriteEngine On' . "\n\t" . 'RewriteCond %{HTTP:Accept-Encoding} (^|\\W)gzip(\\W|$)' . "\n\t" . 'RewriteCond %{REQUEST_FILENAME} \\.(css|js)$' . "\n\t" . 'RewriteCond %{REQUEST_FILENAME}.gz -f' . "\n\t" . 'RewriteRule ^(.*)\\.(css|js)$ $1\\.$2\\.gz [QSA]' . "\n\t" . 'RewriteRule \\.css\\.gz$ - [T=text/css,E=no-gzip:1,E=no-brotli:1]' . "\n\t" . 'RewriteRule \\.js\\.gz$ - [T=application/javascript,E=no-gzip:1,E=no-brotli:1]' . "\n" .
					'</IfModule>' . "\n" .
					'<IfModule mod_headers.c>' . "\n\t" . '<FilesMatch \\.(js|css)\\.gz$>' . "\n\t\t" . 'Header set Content-Encoding gzip' . "\n\t\t" . 'Header merge Vary Accept-Encoding' . "\n" .
					( Gen::GetArrField( $sett, array( 'hdrTrace' ), false ) ? "\t\t" . 'Header set X-seraph-accel-Cache "1.8.7; state=precompressed;"' . "\n" : '' ) .
					"\t" . '</FilesMatch>' . "\n" .
					'</IfModule>' . "\n" .
					'';
			}
		}

		$htaccessBlock = trim( $htaccessBlock );

		if( Gen::HtAccess_GetBlock( 'seraphinite-accelerator' ) != $htaccessBlock )
			$hr = Gen::HrAccom( $hr, Gen::HtAccess_SetBlock( 'seraphinite-accelerator', $htaccessBlock, 5 ) );
	}

	return( $hr );
}

function GetCacheStatusInfo( $siteId, $cbCancel )
{
	$info = CacheGetInfo( $siteId, $cbCancel );
	if( !$info )
		return( null );

	return( $info );
}

function GetAdvCacheFileContent( $sett )
{
	$content = '<?php' . "\n";
	$content .= '/* seraphinite-accelerator */' . "\n";

	$sitesIds = array();
	if( Gen::DoesFuncExist( 'get_sites' ) && is_multisite() )
	{
		foreach( get_sites() as $site )
		{
			switch_to_blog( $site -> blog_id );

			$addrSite = strtolower( Net::GetUrlWithoutProto( Gen::SetLastSlash( get_site_url(), false ) ) );
			if( defined( 'SITE_ID_CURRENT_SITE' ) && defined( 'BLOG_ID_CURRENT_SITE' ) && $site -> blog_id == SITE_ID_CURRENT_SITE && $site -> site_id == BLOG_ID_CURRENT_SITE )
				$siteId = 'm';
			else
				$siteId = '' . $site -> site_id . '_' . $site -> blog_id;
			$sitesIds[ $addrSite ] = $siteId;

			Plugin::SettCacheClear();
			$settSite = Plugin::SettGet();
			$content .= 'function _seraph_accel_siteSettInlineDetach_' . $siteId . '(){ return ' . var_export( $settSite, true ) . '; }' . "\n";

			restore_current_blog();
		}

		$content .= 'function seraph_accel_siteSettInlineDetach($siteId){ $fn = \'_seraph_accel_siteSettInlineDetach_\' . $siteId; return function_exists($fn) ? call_user_func($fn) : null; }' . "\n";
	}
	else
	{
		$content .= 'function seraph_accel_siteSettInlineDetach($siteId){ return ' . var_export( $sett, true ) . '; }' . "\n";
		$sitesIds[ Net::GetUrlWithoutProto( Gen::SetLastSlash( get_site_url(), false ) ) ] = 'm';
	}

	$content .= '$seraph_accel_sites = ' . var_export( $sitesIds, true ) . ';' . "\n";

	$content .= '@include(WP_CONTENT_DIR . \'/plugins/seraphinite-accelerator/cache.php\');' . "\n";
	$content .= '?>';
	return( $content );
}

function GetLoadAvg()
{
	$loadavg = explode( ' ', @file_get_contents( '/proc/loadavg' ) );
	$loadavg = $loadavg[ 0 ];
	return( $loadavg ? intval( $loadavg ) : 0 );
}

function UpdateClientSessId( $curUserId, $token = null, $expirationNew = null )
{
	$siteId = 'm';
	$tmCur = Gen::GetCurRequestTime();

	$sessInfo = GetCacheCurUserSession( $siteId );
	$sessId = @$sessInfo[ 'sessId' ];

	if( $curUserId )
	{
		if( @$sessInfo[ 'userSessId' ] != $token || @$sessInfo[ 'expiration' ] != $expirationNew || @$sessInfo[ 'userId' ] != $curUserId )
		{
			if( Gen::IsEmpty( $sessId ) )
				$sessId = wp_generate_password( 43, false, false );
			SetCacheCurUserSession( $siteId, $sessId, $token, $curUserId, $expirationNew );
		}
	}
	else if( Gen::IsEmpty( $sessId ) )
	{
		$sessId = wp_generate_password( 43, false, false );
		SetCacheCurUserSession( $siteId, $sessId, '0', 0, $tmCur + 12 * HOUR_IN_SECONDS );
	}
	else if( @$sessInfo[ 'userId' ] || @$sessInfo[ 'expiration' ] < $tmCur )
		SetCacheCurUserSession( $siteId, $sessId, '0', 0, $tmCur + 12 * HOUR_IN_SECONDS );
}

